# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for Workstation Config related commands."""


import argparse
import textwrap

from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


def AddAsyncFlag(parser):
  """Adds --async flag."""
  base.ASYNC_FLAG.AddToParser(parser)


def AddWorkstationUpdatePersistentDirectoryFields(parser):
  """Adds persistent directory flags for updating a workstation."""
  group = parser.add_group(help='Persistent directory settings.')
  group.add_argument(
      '--pd-disk-size',
      type=int,
      help=(
          'Specifies the desired size in GB of the persistent directory '
          'mounted at /home.'
      ),
  )


def AddAllowUnauthenticatedCorsPreflightRequestsFlag(parser):
  """Adds a --allow-unauthenticated-cors-preflight-requests flag to the given parser."""
  help_text = """\
    By default, the workstations service makes sure that all requests to the
    workstation are authenticated. CORS preflight requests do
    not include cookies or custom headers, and so are considered
    unauthenticated and blocked by the workstations service. Enabling this
    option allows these unauthenticated CORS preflight requests through to
    the workstation, where it becomes the responsibility of the destination
    server in the workstation to validate the request
  """
  parser.add_argument(
      '--allow-unauthenticated-cors-preflight-requests',
      action='store_true',
      help=help_text,
  )


def AddDisallowUnauthenticatedCorsPreflightRequestsToggleFlag(parser):
  """Adds a --disallow-unauthenticated-cors-preflight-requests flag to the given parser."""
  help_text = """\
    By default, the workstations service makes sure that all requests to the
    workstation are authenticated. CORS preflight requests do
    not include cookies or custom headers, and so are considered
    unauthenticated and blocked by the workstations service. Enabling this
    option allows these unauthenticated CORS preflight requests through to
    the workstation, where it becomes the responsibility of the destination
    server in the workstation to validate the request
  """

  group = parser.add_mutually_exclusive_group()
  group.add_argument(
      '--allow-unauthenticated-cors-preflight-requests',
      action='store_true',
      help=help_text,
  )

  help_text = """\
  If set, requires that all requests to the workstation are authenticated."""
  group.add_argument(
      '--disallow-unauthenticated-cors-preflight-requests',
      action='store_true',
      help=help_text,
  )


def AddDisableLocalhostReplacementFlag(parser):
  """Adds a --disable-localhost-replacement flag to the given parser."""
  help_text = """\
    By default, the workstations service replaces references to localhost,
    127.0.0.1, and 0.0.0.0 with the workstation's hostname in http responses
    from the workstation so that applications under development run properly
    on the workstation. This may intefere with some applications, and so
    this option allows that behavior to be disabled.
  """
  parser.add_argument(
      '--disable-localhost-replacement',
      action='store_true',
      help=help_text,
  )


def AddDisableLocalhostReplacementToggleFlag(parser):
  """Adds a --enable-localhost-replacement flag to the given parser."""
  help_text = """\
    By default, the workstations service replaces references to localhost,
    127.0.0.1, and 0.0.0.0 with the workstation's hostname in http responses
    from the workstation so that applications under development run properly
    on the workstation. This may intefere with some applications, and so
    this option allows that behavior to be disabled.
  """

  group = parser.add_mutually_exclusive_group()
  group.add_argument(
      '--disable-localhost-replacement',
      action='store_true',
      help=help_text,
  )

  help_text = """\
  If set, requires that all requests to the workstation are authenticated."""
  group.add_argument(
      '--enable-localhost-replacement',
      action='store_true',
      help=help_text,
  )


def AddAllowedPortsFlag(parser):
  """Adds a --allowed-ports flag to the given parser."""
  help_text = """\
  A Single or Range of ports externally accessible in the workstation.
  If not specified defaults to ports 22, 80 and ports 1024-65535.

  To specify a single port, both first and last should be same.

  Example:

    $ {command} --allowed-ports=first=9000,last=9090
    $ {command} --allowed-ports=first=80,last=80"""
  parser.add_argument(
      '--allowed-ports',
      metavar='ALLOWED_PORTS',
      type=arg_parsers.ArgObject(
          spec={
              'first': int,
              'last': int,
          },
          required_keys=['first', 'last'],
      ),
      action=arg_parsers.FlattenAction(),
      help=help_text,
  )


def LocationsAttributeConfig(
    location_fallthrough=False, global_fallthrough=False
):
  """Create a location attribute in resource argument.

  Args:
    location_fallthrough: If set, enables fallthroughs for the location
      attribute.
    global_fallthrough: If set, enables global fallthroughs for the location
      attribute.

  Returns:
    Location resource argument parameter config
  """
  fallthroughs = []
  if location_fallthrough:
    fallthroughs.append(
        deps.PropertyFallthrough(properties.VALUES.workstations.region)
    )
  if global_fallthrough:
    fallthroughs.append(
        deps.Fallthrough(lambda: '-', hint='default is all regions')
    )
  return concepts.ResourceParameterAttributeConfig(
      name='region',
      fallthroughs=fallthroughs,
      help_text='The region for the {resource}.',
  )


def ClustersAttributeConfig(
    cluster_fallthrough=False, global_fallthrough=False
):
  """Create a cluster attribute in resource argument.

  Args:
    cluster_fallthrough: If set, enables fallthroughs for the cluster attribute.
    global_fallthrough: If set, enables global fallthroughs for the cluster
      attribute.

  Returns:
    Cluster resource argument parameter config
  """
  fallthroughs = []
  if cluster_fallthrough:
    fallthroughs.append(
        deps.PropertyFallthrough(properties.VALUES.workstations.cluster)
    )
  if global_fallthrough:
    fallthroughs.append(
        deps.Fallthrough(
            lambda: '-',
            hint='default is all clusters',
        )
    )
  return concepts.ResourceParameterAttributeConfig(
      name='cluster',
      fallthroughs=fallthroughs,
      help_text='The cluster for the {resource}.',
  )


def ConfigsAttributeConfig(config_fallthrough=False, global_fallthrough=False):
  """Create a config attribute in resource argument.

  Args:
    config_fallthrough: If set, enables fallthroughs for the config attribute
      using the value set in workstations/config.
    global_fallthrough: If set, enables global fallthroughs for the config
      attribute.

  Returns:
    Config resource argument parameter config
  """
  fallthroughs = []
  if config_fallthrough:
    fallthroughs.append(
        deps.PropertyFallthrough(properties.VALUES.workstations.config)
    )
  if global_fallthrough:
    fallthroughs.append(
        deps.Fallthrough(
            lambda: '-',
            hint='default is all configs',
        )
    )
  return concepts.ResourceParameterAttributeConfig(
      name='config',
      fallthroughs=fallthroughs,
      help_text='The config for the {resource}.',
  )


def WorkstationsAttributeConfig():
  """Create a workstation attribute in resource argument.

  Returns:
    Workstation resource argument parameter config
  """
  return concepts.ResourceParameterAttributeConfig(
      name='workstation',
      help_text='The workstation.',
  )


def AddConfigResourceArg(
    parser, api_version='v1beta', flag_anchor=False, global_fallthrough=False
):
  """Create a config resource argument."""
  spec = concepts.ResourceSpec(
      'workstations.projects.locations.workstationClusters.workstationConfigs',
      resource_name='config',
      api_version=api_version,
      workstationConfigsId=ConfigsAttributeConfig(
          global_fallthrough=global_fallthrough
      ),
      workstationClustersId=ClustersAttributeConfig(
          cluster_fallthrough=True, global_fallthrough=global_fallthrough
      ),
      locationsId=LocationsAttributeConfig(
          location_fallthrough=True, global_fallthrough=global_fallthrough
      ),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False,
  )
  concept_parsers.ConceptParser.ForResource(
      '--config' if flag_anchor else 'config',
      spec,
      'The group of arguments defining a config',
      required=True,
  ).AddToParser(parser)


def AddWorkstationResourceArg(parser, api_version='v1beta'):
  """Create a workstation resource argument."""
  spec = concepts.ResourceSpec(
      'workstations.projects.locations.workstationClusters.workstationConfigs.workstations',
      resource_name='workstation',
      api_version=api_version,
      workstationsId=WorkstationsAttributeConfig(),
      workstationConfigsId=ConfigsAttributeConfig(config_fallthrough=True),
      workstationClustersId=ClustersAttributeConfig(cluster_fallthrough=True),
      locationsId=LocationsAttributeConfig(location_fallthrough=True),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )
  concept_parsers.ConceptParser.ForResource(
      'workstation',
      spec,
      'The group of arguments defining a workstation',
      required=True,
  ).AddToParser(parser)


def AddIdleTimeoutFlag(parser, use_default=True):
  """Adds an --idle-timeout flag to the given parser."""
  help_text = """\
  How long (in seconds) to wait before automatically stopping an instance that
  hasn't received any user traffic. A value of 0 indicates that this instance
  should never time out due to idleness.
  """
  parser.add_argument(
      '--idle-timeout',
      default=7200 if use_default else None,
      type=int,
      help=help_text,
  )


def AddRunningTimeoutFlag(parser, use_default=True):
  """Adds an --running-timeout flag to the given parser."""
  help_text = """\
  How long (in seconds) to wait before automatically stopping a workstation
  after it started. A value of 0 indicates that workstations using this config
  should never time out.
  """
  parser.add_argument(
      '--running-timeout',
      default=7200 if use_default else None,
      type=int,
      help=help_text,
  )


def AddMachineTypeFlag(parser, use_default=True):
  """Adds a --machine-type flag to the given parser."""
  help_text = """\
  Machine type determines the specifications of the Compute Engine machines
  that the workstations created under this configuration will run on."""
  parser.add_argument(
      '--machine-type',
      type=str,
      default='e2-standard-4' if use_default else None,
      help=help_text,
  )


def AddServiceAccountFlag(parser):
  """Adds a --service-account flag to the given parser."""
  help_text = """\
  Email address of the service account that will be used on VM instances used to
  support this config. This service account must have permission to pull the
  specified container image. If not set, VMs will run without a service account,
  in which case the image must be publicly accessible."""
  parser.add_argument('--service-account', help=help_text)


def AddServiceAccountScopes(parser):
  """Adds a --service-account-scopes flag to the given parser."""
  help_text = """\
  Scopes to grant to the service_account. Various scopes are
  automatically added based on feature usage. When specified, users of
  workstations under this configuration must have
  iam.serviceAccounts.actAs` on the service account.
  """
  parser.add_argument(
      '--service-account-scopes',
      metavar='SERVICE_ACCOUNT_SCOPES',
      type=arg_parsers.ArgList(),
      help=help_text,
  )


def AddNetworkTags(parser):
  """Adds a --network-tags flag to the given parser."""
  help_text = """\
  Network tags to add to the Google Compute Engine machines backing the
  Workstations.

  Example:

    $ {command} --network-tags=tag_1,tag_2
  """
  parser.add_argument(
      '--network-tags',
      metavar='NETWORK_TAGS',
      type=arg_parsers.ArgList(),
      help=help_text,
  )


def AddPoolSize(parser, use_default=True):
  """Adds a --pool-size flag to the given parser."""
  help_text = """\
  Number of instances to pool for faster Workstation startup."""
  parser.add_argument(
      '--pool-size',
      default=0 if use_default else None,
      type=int,
      help=help_text,
  )


def AddDisablePublicIpAddresses(parser, use_default=True):
  """Adds a --disable-public-ip-addresses flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have no public IP address."""
  parser.add_argument(
      '--disable-public-ip-addresses',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddDisableTcpConnections(parser, use_default=True):
  """Adds a --disable-tcp-connections flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, workstations don't allow plain TCP connections."""
  parser.add_argument(
      '--disable-tcp-connections',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddEnableTcpConnections(parser):
  """Adds a --enable-tcp-connections flag to the given parser."""
  help_text = """\
  If set, workstations allow plain TCP connections."""

  group = parser.add_mutually_exclusive_group()
  group.add_argument(
      '--enable-tcp-connections',
      action='store_true',
      help=help_text,
  )

  help_text = """\
  If set, workstations don't allow plain TCP connections."""
  group.add_argument(
      '--disable-tcp-connections',
      action='store_true',
      help=help_text,
  )


def AddShieldedSecureBoot(parser, use_default=True):
  """Adds --shielded-secure-boot flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have Secure Boot enabled."""
  parser.add_argument(
      '--shielded-secure-boot',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddShieldedVtpm(parser, use_default=True):
  """Adds a --shielded-vtpm flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have vTPM enabled."""
  parser.add_argument(
      '--shielded-vtpm',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddShieldedIntegrityMonitoring(parser, use_default=True):
  """Adds a --shielded-integrity-monitoring flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have integrity monitoring enabled."""
  parser.add_argument(
      '--shielded-integrity-monitoring',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddSourceWorkstation(parser):
  """Adds a --source-workstation flag to the given parser."""
  help_text = """\
  Source workstation from which this workstations persistent directories are
  cloned on creation. When specified, the workstations service agent must have
  `compute.disks.createSnapshot` and `compute.snapshots.useReadOnly` permissions
  on the source workstation's host project.
  """
  parser.add_argument(
      '--source-workstation',
      type=str,
      help=help_text,
  )


def AddEnableAuditAgent(parser, use_default=True):
  """Adds an --enable-audit-agent flag to the given parser."""
  help_text = """\
  Whether to enable Linux `auditd` logging on the workstation. When enabled,
  a service account must also be specified that has `logging.buckets.write`
  permission on the project."""
  parser.add_argument(
      '--enable-audit-agent',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddGrantWorkstationAdminRoleOnCreate(parser, use_default=True):
  """Adds a --grant-workstation-admin-role-on-create flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, creator of a workstation will get `roles/workstations.policyAdmin`
  role along with `roles/workstations.user` role on the workstation created by
  them."""
  parser.add_argument(
      '--grant-workstation-admin-role-on-create',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddEnableConfidentialCompute(parser, use_default=True):
  """Adds an --enable-confidential-compute flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have confidential compute enabled."""
  parser.add_argument(
      '--enable-confidential-compute',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddEnableNestedVirtualization(parser, use_default=True):
  """Adds an --enable-nested-virtualization flag to the given parser."""
  help_text = """\
  Default value is false.
  If set, instances will have nested virtualization enabled."""
  parser.add_argument(
      '--enable-nested-virtualization',
      action='store_true',
      default=False if use_default else None,
      help=help_text,
  )


def AddBootDiskSize(parser, use_default=True):
  """Adds a --boot-disk-size flag to the given parser."""
  help_text = """\
  Size of the boot disk in GB."""
  parser.add_argument(
      '--boot-disk-size',
      default=50 if use_default else None,
      type=int,
      help=help_text,
  )


def AddPdDiskType(parser):
  """Adds a --pd-disk-type flag to the given parser."""
  help_text = """\
  Type of the persistent directory."""
  parser.add_argument(
      '--pd-disk-type',
      choices=[
          'pd-standard',
          'pd-balanced',
          'pd-ssd',
      ],
      default='pd-standard',
      help=help_text,
  )


def AddNoPersistentStorageOrPdOrDisk(parser):
  """Adds a --no-persistent-storage or group of persistent directory flags to the given parser."""
  top_level_mutex_group = parser.add_mutually_exclusive_group()

  help_text = """\
  If set, workstations under this configuration will not have a persistent directory."""
  top_level_mutex_group.add_argument(
      '--no-persistent-storage',
      action='store_true',
      help=help_text,
  )

  disk_group = top_level_mutex_group.add_group()
  AddDiskTypeFlag(disk_group)
  AddDiskReclaimPolicyFlag(disk_group)
  AddDiskArchiveTimeoutFlag(disk_group)
  disk_size_snapshot_group = disk_group.add_mutually_exclusive_group()
  AddDiskSizeFlag(disk_size_snapshot_group)
  AddDiskSnapshotFlag(disk_size_snapshot_group)
  pd_group = top_level_mutex_group.add_group()
  AddPdDiskType(pd_group)
  AddPdReclaimPolicy(pd_group)
  AddPdDiskSizeOrSnapshot(pd_group)


def AddPdDiskSizeOrSnapshot(parser):
  """Adds a --pd-disk-size or --pd-source-snapshot flag to the given parser."""
  help_text = """\
  Size of the persistent directory in GB."""
  group = parser.add_mutually_exclusive_group()
  group.add_argument(
      '--pd-disk-size',
      choices=[10, 50, 100, 200, 500, 1000],
      default=200,
      type=int,
      help=help_text,
  )

  help_text = """\
  Name of the snapshot to use as the source for the home disk."""
  group.add_argument(
      '--pd-source-snapshot',
      default='',
      help=help_text,
  )


def AddPdDiskTypeArg():
  help_text = """\
  Type of the persistent directory."""
  return base.Argument(
      '--pd-disk-type',
      choices=[
          'pd-standard',
          'pd-balanced',
          'pd-ssd',
      ],
      default='pd-standard',
      help=help_text,
  )


def AddPdDiskSizeArg(use_default):
  help_text = """\
  Size of the persistent directory in GB."""
  return base.Argument(
      '--pd-disk-size',
      choices=[10, 50, 100, 200, 500, 1000],
      default=200 if use_default else None,
      type=int,
      help=help_text,
  )


def AddPdSourceSnapshotArg():
  help_text = """\
  Name of the snapshot to use as the source for the persistent directory."""
  return base.Argument(
      '--pd-source-snapshot',
      default='',
      help=help_text,
  )


def AddPersistentDirectoriesOrHyperdisks(parser, use_default=True):
  """Adds a --pd-disk-size, --pd-disk-type, and --pd-source-snapshot flag to the given parser."""
  AddDiskArchiveTimeoutFlag(parser)
  group = parser.add_mutually_exclusive_group()
  # OPTION 1: "--pd-source-snapshot"
  AddPdSourceSnapshotArg().AddToParser(group)
  # OPTION 2: "--pd-disk-type" and "--pd-disk-size"
  pd_type_size_group = group.add_group(mutex=False)
  AddPdDiskTypeArg().AddToParser(pd_type_size_group)
  AddPdDiskSizeArg(use_default).AddToParser(pd_type_size_group)
  # OPTION 3: "--disk-type" and "--disk-size"
  disk_type_size_group = group.add_group(mutex=False)
  AddDiskTypeFlag(disk_type_size_group)
  AddDiskSizeFlag(disk_type_size_group)
  # OPTION 4: "--disk-source-snapshot"
  AddDiskSnapshotFlag(group)


def AddPdReclaimPolicy(parser):
  """Adds a --pd-reclaim-policy flag to the given parser."""
  help_text = """\
  What should happen to the disk after the Workstation is deleted."""
  parser.add_argument(
      '--pd-reclaim-policy',
      choices={
          'delete': 'The persistent disk will be deleted with the Workstation.',
          'retain': (
              'The persistent disk will be remain after the workstation is'
              ' deleted and the administrator must manually delete the disk.'
          ),
      },
      default='delete',
      help=help_text,
  )


def AddDiskTypeFlag(parser):
  """Adds a --disk-type flag to the given parser."""
  help_text = """\
  Type of the persistent directory."""
  parser.add_argument(
      '--disk-type',
      choices=[
          'pd-standard',
          'pd-balanced',
          'pd-ssd',
          'hyperdisk-balanced-ha',
      ],
      help=help_text,
  )


def AddDiskReclaimPolicyFlag(parser):
  """Adds a --disk-reclaim-policy flag to the given parser."""
  help_text = """\
  What should happen to the disk after the Workstation is deleted."""
  parser.add_argument(
      '--disk-reclaim-policy',
      choices={
          'delete': 'The persistent disk will be deleted with the Workstation.',
          'retain': (
              'The persistent disk will be remain after the workstation is'
              ' deleted and the administrator must manually delete the disk.'
          ),
      },
      default='delete',
      help=help_text,
  )


def AddDiskSizeFlag(parser):
  """Adds a --disk-size flag to the given parser."""
  help_text = """\
  Size of the persistent directory in GB."""
  parser.add_argument(
      '--disk-size',
      choices=[10, 50, 100, 200, 500, 1000],
      type=int,
      help=help_text,
  )


def AddDiskSnapshotFlag(parser):
  """Adds a --disk-source-snapshot flag to the given parser."""
  help_text = """\
  Name of the snapshot to use as the source for the home disk."""
  parser.add_argument(
      '--disk-source-snapshot',
      default='',
      help=help_text,
  )


def AddDiskArchiveTimeoutFlag(parser):
  """Adds a --disk-archive-timeout flag to the given parser."""
  help_text = """\
  Number of seconds to wait after initially creating or subsequently
  shutting down the workstation before converting its disk into a
  snapshot to save costs. A value of 0 indicates that the disk will never be
  archived.
  """
  parser.add_argument(
      '--disk-archive-timeout',
      type=int,
      help=help_text,
  )


def AddEphemeralDirectory(parser):
  """Adds a --ephemeral-directory flag to the given parser."""
  spec = {
      'mount-path': str,
      'disk-type': str,
      'source-snapshot': str,
      'source-image': str,
      'read-only': arg_parsers.ArgBoolean(),
  }
  help_text = """\
  Ephemeral directory which won't persist across workstation sessions. An ephemeral directory is backed by a Compute Engine persistent disk whose mount-path, source-snapshot, source-image, and read-only are configurable.

  *mount-path*::: Location of this directory in the running workstation.

  *source-snapshot:: Name of the snapshot to use as the source for the disk. Must be empty if [source_image][] is set. Must be empty if [read_only][] is false. Updating [source_snapshot][] will update content in the ephemeral directory after the workstation is restarted.

  *source-image::: Name of the disk image to use as the source for the disk. Must be empty if [source_snapshot][] is set. Updating [source_image][] will update content in the ephemeral directory after the workstation is restarted.

  *read-only::: Whether the disk is read only. If true, the disk may be shared by multiple VMs and [source_snapshot][] must be set. Set to false when not specified and true when specified.

  Example:

    $ {command} --ephemeral-directory="mount-path=/home2,disk-type=pd-balanced,source-snapshot=projects/my-project/global/snapshots/snapshot,read-only=true"
  """
  parser.add_argument(
      '--ephemeral-directory',
      type=arg_parsers.ArgDict(spec=spec),
      action='append',
      metavar='PROPERTY=VALUE',
      help=help_text,
  )


def AddContainerImageField(parser, use_default=True):
  """Adds the --container-predefined-image and --container-custom-image flags to the given parser."""
  predefined_image_help_text = """\
  Code editor on base images."""
  custom_image_help_text = """\
  A docker image for the workstation. This image must be accessible by the
  service account configured in this configuration (--service-account). If no
  service account is defined, this image must be public.
  """
  group = parser.add_mutually_exclusive_group()
  group.add_argument(
      '--container-predefined-image',
      choices={
          'codeoss': 'Code OSS',
          'intellij': 'IntelliJ IDEA Ultimate',
          'pycharm': 'PyCharm Professional',
          'rider': 'Rider',
          'webstorm': 'WebStorm',
          'phpstorm': 'PhpStorm',
          'rubymine': 'RubyMine',
          'goland': 'GoLand',
          'clion': 'CLion',
          'base-image': 'Base image - no IDE',
          'codeoss-cuda': 'Code OSS + CUDA toolkit',
      },
      default='codeoss' if use_default else None,
      help=predefined_image_help_text,
  )

  group.add_argument(
      '--container-custom-image', type=str, help=custom_image_help_text
  )


def AddContainerCommandField(parser):
  """Adds a --container-command flag to the given parser."""
  help_text = """\
  If set, overrides the default ENTRYPOINT specified by the image.

  Example:

    $ {command} --container-command=executable,parameter_1,parameter_2"""
  parser.add_argument(
      '--container-command',
      metavar='CONTAINER_COMMAND',
      type=arg_parsers.ArgList(),
      help=help_text,
  )


def AddContainerArgsField(parser):
  """Adds a --container-args flag to the given parser."""
  help_text = """\
  Arguments passed to the entrypoint.

  Example:

    $ {command} --container-args=arg_1,arg_2"""
  parser.add_argument(
      '--container-args',
      metavar='CONTAINER_ARGS',
      type=arg_parsers.ArgList(),
      help=help_text,
  )


def AddContainerEnvField(parser):
  """Adds a --container-env flag to the given parser."""
  help_text = """\
  Environment variables passed to the container.

  Example:

    $ {command} --container-env=key1=value1,key2=value2"""
  parser.add_argument(
      '--container-env',
      metavar='CONTAINER_ENV',
      type=arg_parsers.ArgDict(key_type=str, value_type=str),
      help=help_text,
  )


def AddContainerWorkingDirField(parser):
  """Adds a --container-working-dir flag to the given parser."""
  help_text = """\
  If set, overrides the default DIR specified by the image."""
  parser.add_argument('--container-working-dir', help=help_text)


def AddContainerRunAsUserField(parser):
  """Adds a --container-run-as-user flag to the given parser."""
  help_text = """\
  If set, overrides the USER specified in the image with the given
  uid."""
  parser.add_argument('--container-run-as-user', type=int, help=help_text)


def AddWorkstationPortField(parser):
  """Adds a workstation-port flag to the given parser."""
  help_text = """\
  The port on the workstation to which traffic should be sent."""
  parser.add_argument('workstation_port', type=int, help=help_text)


def AddPortField(parser):
  """Adds a --port flag to the given parser."""
  help_text = """\
  The port on the workstation to which traffic should be sent."""
  parser.add_argument('--port', type=int, default=22, help=help_text)


def AddLocalHostPortField(parser):
  """Adds a --local-host-port flag to the given parser."""
  help_text = """\
  `LOCAL_HOST:LOCAL_PORT` on which gcloud should bind and listen for connections
  that should be tunneled.

  `LOCAL_PORT` may be omitted, in which case it is treated as 0 and an arbitrary
  unused local port is chosen. The colon also may be omitted in that case.

  If `LOCAL_PORT` is 0, an arbitrary unused local port is chosen."""
  parser.add_argument(
      '--local-host-port',
      type=arg_parsers.HostPort.Parse,
      default='localhost:0',
      help=help_text,
  )


def AddStartWorkstationFlag(parser):
  """Adds a --start-workstation flag to the given parser."""
  help_text = """\
  If set, automatically starts the workstation if it is currently stopped."""
  parser.add_argument(
      '--start-workstation',
      action='store_true',
      help=help_text,
  )


def AddCommandField(parser):
  """Adds a --command flag to the given parser."""
  help_text = """\
      A command to run on the workstation.

      Runs the command on the target workstation and then exits.
      """
  parser.add_argument('--command', type=str, help=help_text)


def AddSshArgsAndUserField(parser):
  """Additional flags and positional args to be passed to *ssh(1)*."""
  help_text = """\
  The username with which to SSH.
  """
  parser.add_argument('--user', type=str, default='user', help=help_text)

  help_text = """\
  Flags and positionals passed to the underlying ssh implementation."""
  parser.add_argument('ssh_args', nargs=argparse.REMAINDER, help=help_text)

  help_text = """\
  Additional flags to be passed to *ssh(1)*. It is recommended that flags
  be passed using an assignment operator and quotes. Example:

    $ {command} --ssh-flag="-vvv" --ssh-flag="-L 80:localhost:80"
  """
  parser.add_argument('--ssh-flag', action='append', help=help_text)


def AddEncryptionKeyFields(parser):
  """Adds the --kms-key and --kms-key-service-account flags to the given parser."""
  group = parser.add_group(help='Encryption key settings')

  help_text = """\
  The customer-managed encryption key to use for this config. If not specified,
  a Google-managed encryption key is used.
  """
  group.add_argument('--kms-key', type=str, help=help_text, required=True)

  help_text = """\
  The service account associated with the provided customer-managed encryption
  key.
  """
  group.add_argument('--kms-key-service-account', type=str, help=help_text)


def AddLabelsField(parser):
  """Adds a --labels flag to the given parser."""
  help_text = """\
  Labels that are applied to the configuration and propagated to the underlying
  Compute Engine resources.

  Example:

    $ {command} --labels=label1=value1,label2=value2"""
  parser.add_argument(
      '--labels',
      metavar='LABELS',
      type=arg_parsers.ArgDict(key_type=str, value_type=str),
      help=help_text,
  )


def AddAcceleratorFields(parser):
  """Adds the --accelerator-type and --accelerator-count flags to the given parser."""
  group = parser.add_group(help='Accelerator settings')

  help_text = """\
  The type of accelerator resource to attach to the instance, for example,
  "nvidia-tesla-p100".
  """
  group.add_argument('--accelerator-type', type=str, help=help_text)

  help_text = """\
  The number of accelerator cards exposed to the instance.
  """
  group.add_argument(
      '--accelerator-count', type=int, help=help_text, required=True
  )


def AddBoostConfigs(parser):
  """Adds a --boost-config flag to the given parser."""
  help_text = """\
  Boost Configuration(s) that workstations running with this configuration can
  boost up to. This includes id (required), machine-type, accelerator-type,
  accelerator-count, pool-size, boot-disk-size, and enable-nested-virtualization.

  Example:

    $ {command} --boost-config=id=boost1,machine-type=n1-standard-4,accelerator-type=nvidia-tesla-t4,accelerator-count=1"""
  parser.add_argument(
      '--boost-config',
      metavar='BOOST_CONFIG',
      type=arg_parsers.ArgObject(
          spec={
              'id': str,
              'machine-type': str,
              'accelerator-type': str,
              'accelerator-count': int,
              'pool-size': int,
              'boot-disk-size': int,
              'enable-nested-virtualization': bool,
              'reservation-affinity': arg_parsers.ArgObject(
                  spec={
                      'key': str,
                      'values': arg_parsers.ArgList(),
                      'consume-reservation-type': (
                          ValidateConsumeReservationType
                      ),
                  }
              ),
          },
          required_keys=['id'],
      ),
      action=arg_parsers.FlattenAction(),
      help=help_text,
  )


def AddBoost(parser):
  """Adds a --boost flag to the given parser."""
  help_text = """\
  Id of a boost configuration to start a workstations with.

  Example:

    $ {command} --boost=boost1"""
  parser.add_argument(
      '--boost',
      metavar='BOOST',
      type=str,
      help=help_text,
  )


def AddReplicaZones(parser):
  """Adds a --replica-zones flag to the given parser."""
  help_text = """\
  Specifies the zones the VM and disk resources will be
  replicated within the region. If set, exactly two zones within the
  workstation cluster's region must be specified.

  Example:

    $ {command} --replica-zones=us-central1-a,us-central1-f
  """
  parser.add_argument(
      '--replica-zones',
      metavar='REPLICA_ZONES',
      type=arg_parsers.ArgList(),
      help=help_text,
  )


def AddDisableSSHToVM(parser):
  """Adds a --disable-ssh-to-vm flag to the given parser."""
  group = parser.add_mutually_exclusive_group()
  help_text = """\
  If set, workstations disable SSH connections to the root VM."""
  group.add_argument(
      '--disable-ssh-to-vm',
      action='store_true',
      help=help_text,
  )
  help_text = """\
  If set, workstations enable SSH connections to the root VM."""
  group.add_argument(
      '--enable-ssh-to-vm',
      action='store_true',
      help=help_text,
  )


def AddEnableSSHToVM(parser, use_default=False):
  """Adds a --enable-ssh-to-vm flag to the given parser."""
  help_text = """\
  Default value is False.
  If set, workstations enable SSH connections to the root VM."""
  parser.add_argument(
      '--enable-ssh-to-vm',
      default=False if use_default else None,
      action='store_true',
      help=help_text,
  )


def AddDeprecatedDisableSSHToVM(parser, use_default=True):
  """Adds a --disable-ssh-to-vm flag to the given parser."""
  help_text = """\
  Default value is False.
  If set, workstations disable SSH connections to the root VM."""
  parser.add_argument(
      '--disable-ssh-to-vm',
      action=actions.DeprecationAction(
          '--disable-ssh-to-vm',
          warn=(
              'The {flag_name} option is deprecated; use --enable-ssh-to-vm'
              ' instead.'
          ),
          removed=False,
          action='store_true',
      ),
      default=False if use_default else False,
      help=help_text,
  )


def AddVmTags(parser):
  """Adds a --vm-tags flag to the given parser."""
  help_text = """\
  Resource manager tags to be bound to the instance.
  Tag keys and values have the same definition as
  https://cloud.google.com/resource-manager/docs/tags/tags-overview

  Example:

    $ {command} --vm-tags=tagKeys/key1=tagValues/value1,tagKeys/key2=tagValues/value2"""
  parser.add_argument(
      '--vm-tags',
      metavar='VM_TAGS',
      type=arg_parsers.ArgDict(key_type=str, value_type=str),
      help=help_text,
  )


def AddMaxUsableWorkstationsCount(parser):
  """Adds a --max-usable-workstations-count flag to the given parser."""
  help_text = """\
  Maximum number of workstations under this configuration a user can have
  `workstations.workstation.use` permission on.

  If not specified, defaults to `0`, which indicates a user can have unlimited
  number of workstations under this configuration."""

  parser.add_argument(
      '--max-usable-workstations-count',
      metavar='MAX_USABLE_WORKSTATIONS_COUNT',
      type=int,
      default=0,
      help=help_text,
  )


def AddReservationAffinity(parser):
  """Adds a --reservation-affinity flag to the given parser."""
  help_text = """\
  Reservation Affinity for the VM. This includes key, values, and
  consumeReservationType.

  Example:
    $ {command} --reservation-affinity=key=compute.googleapis.com/reservation-name,consumeReservationType=SPECIFIC_RESERVATION,values=my-reservation
  """

  parser.add_argument(
      '--reservation-affinity',
      metavar='RESERVATION_AFFINITY',
      type=arg_parsers.ArgObject(
          spec={
              'key': str,
              'values': arg_parsers.ArgList(),
              'consume-reservation-type': ValidateConsumeReservationType,
          },
      ),
      help=help_text,
  )


def AddStartupScriptUri(parser):
  """Adds the --startup-script-uri flag to the given parser."""
  help_text = """\
  Link to the startup script stored in Cloud Storage. The script is executed
  on the workstation VM after it is booted.

  Example:

    $ {command} --startup-script-uri=gs://{bucket-name}/{object-name}
  """
  parser.add_argument(
      '--startup-script-uri', type=str, help=help_text
  )


def AddInstanceMetadata(parser: argparse.ArgumentParser) -> None:
  """Adds the --instance-metadata flag to the given parser."""
  help_text = textwrap.dedent("""\
  Custom metadata to apply to Compute Engine instances.

  Example:

  $ {command} --instance-metadata=key1=value1,key2=value2
  """)
  parser.add_argument(
      '--instance-metadata',
      type=arg_parsers.ArgDict(key_type=str, value_type=str), help=help_text
  )


def ValidateConsumeReservationType(value):
  """Validates the consume-reservation-type value."""
  allowed_values = [
      'ANY_RESERVATION',
      'NO_RESERVATION',
      'SPECIFIC_RESERVATION',
  ]
  if value not in allowed_values:
    raise argparse.ArgumentTypeError(
        f"Invalid choice: '{value}'. Valid choices are {allowed_values}"
    )
  return value



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for getting available accelerator topologies in managed instance groups."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class GetAvailableAcceleratorTopologies(base.DescribeCommand):
  """Get available accelerator topologies for a managed instance group."""

  @staticmethod
  def Args(parser):
    instance_groups_flags.MakeZonalInstanceGroupManagerArg().AddArgument(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    resource_arg = instance_groups_flags.MakeZonalInstanceGroupManagerArg()
    igm_ref = resource_arg.ResolveAsResource(
        args,
        holder.resources,
        default_scope=compute_scope.ScopeEnum.ZONE,
        scope_lister=flags.GetDefaultScopeLister(client),
    )

    service = client.apitools_client.instanceGroupManagers
    request = client.messages.ComputeInstanceGroupManagersGetAvailableAcceleratorTopologiesRequest(
        resourceId=igm_ref.Name(),
        zone=igm_ref.zone,
        project=igm_ref.project,
    )
    return service.GetAvailableAcceleratorTopologies(request)


GetAvailableAcceleratorTopologies.detailed_help = {
    'brief': (
        'Get available accelerator topologies for a managed instance group.'
    ),
    'DESCRIPTION': (
        """\
        *{command}* gets available accelerator topologies for a managed instance group.

        Note: This command is only available to allowlisted projects/organizations
        and requires the managed instance group to be configured with a workload policy
        in raw capacity mode.
        """
    ),
    'EXAMPLES': (
        """\
        To get available accelerator topologies for a managed instance group 'my-group' in zone 'us-central1-a', run:

            $ {command} my-group --zone=us-central1-a
        """
    ),
}



"""Generated client library for apphub version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.apphub.v1alpha import apphub_v1alpha_messages as messages


class ApphubV1alpha(base_api.BaseApiClient):
  """Generated client library for service apphub version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://apphub.googleapis.com/'
  MTLS_BASE_URL = 'https://apphub.mtls.googleapis.com/'

  _PACKAGE = 'apphub'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ApphubV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new apphub handle."""
    url = url or self.BASE_URL
    super(ApphubV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_applications_services = self.ProjectsLocationsApplicationsServicesService(self)
    self.projects_locations_applications_workloads = self.ProjectsLocationsApplicationsWorkloadsService(self)
    self.projects_locations_applications = self.ProjectsLocationsApplicationsService(self)
    self.projects_locations_discoveredServices = self.ProjectsLocationsDiscoveredServicesService(self)
    self.projects_locations_discoveredWorkloads = self.ProjectsLocationsDiscoveredWorkloadsService(self)
    self.projects_locations_extendedMetadataSchemas = self.ProjectsLocationsExtendedMetadataSchemasService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_serviceProjectAttachments = self.ProjectsLocationsServiceProjectAttachmentsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsApplicationsServicesService(base_api.BaseApiService):
    """Service class for the projects_locations_applications_services resource."""

    _NAME = 'projects_locations_applications_services'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsApplicationsServicesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a Service in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsServicesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/services',
        http_method='POST',
        method_id='apphub.projects.locations.applications.services.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId', 'serviceId'],
        relative_path='v1alpha/{+parent}/services',
        request_field='service',
        request_type_name='ApphubProjectsLocationsApplicationsServicesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a Service from an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsServicesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/services/{servicesId}',
        http_method='DELETE',
        method_id='apphub.projects.locations.applications.services.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsServicesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a Service in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsServicesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Service) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/services/{servicesId}',
        http_method='GET',
        method_id='apphub.projects.locations.applications.services.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsServicesGetRequest',
        response_type_name='Service',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Services in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsServicesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListServicesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/services',
        http_method='GET',
        method_id='apphub.projects.locations.applications.services.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/services',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsServicesListRequest',
        response_type_name='ListServicesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a Service in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsServicesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/services/{servicesId}',
        http_method='PATCH',
        method_id='apphub.projects.locations.applications.services.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='service',
        request_type_name='ApphubProjectsLocationsApplicationsServicesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsApplicationsWorkloadsService(base_api.BaseApiService):
    """Service class for the projects_locations_applications_workloads resource."""

    _NAME = 'projects_locations_applications_workloads'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsApplicationsWorkloadsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a Workload in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsWorkloadsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/workloads',
        http_method='POST',
        method_id='apphub.projects.locations.applications.workloads.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId', 'workloadId'],
        relative_path='v1alpha/{+parent}/workloads',
        request_field='workload',
        request_type_name='ApphubProjectsLocationsApplicationsWorkloadsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a Workload from an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsWorkloadsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/workloads/{workloadsId}',
        http_method='DELETE',
        method_id='apphub.projects.locations.applications.workloads.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsWorkloadsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a Workload in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsWorkloadsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Workload) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/workloads/{workloadsId}',
        http_method='GET',
        method_id='apphub.projects.locations.applications.workloads.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsWorkloadsGetRequest',
        response_type_name='Workload',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Workloads in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsWorkloadsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListWorkloadsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/workloads',
        http_method='GET',
        method_id='apphub.projects.locations.applications.workloads.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/workloads',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsWorkloadsListRequest',
        response_type_name='ListWorkloadsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a Workload in an Application.

      Args:
        request: (ApphubProjectsLocationsApplicationsWorkloadsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}/workloads/{workloadsId}',
        http_method='PATCH',
        method_id='apphub.projects.locations.applications.workloads.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='workload',
        request_type_name='ApphubProjectsLocationsApplicationsWorkloadsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsApplicationsService(base_api.BaseApiService):
    """Service class for the projects_locations_applications resource."""

    _NAME = 'projects_locations_applications'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsApplicationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsApplicationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications',
        http_method='POST',
        method_id='apphub.projects.locations.applications.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['applicationId', 'requestId'],
        relative_path='v1alpha/{+parent}/applications',
        request_field='application',
        request_type_name='ApphubProjectsLocationsApplicationsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsApplicationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}',
        http_method='DELETE',
        method_id='apphub.projects.locations.applications.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsApplicationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Application) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}',
        http_method='GET',
        method_id='apphub.projects.locations.applications.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsGetRequest',
        response_type_name='Application',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (ApphubProjectsLocationsApplicationsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}:getIamPolicy',
        http_method='GET',
        method_id='apphub.projects.locations.applications.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1alpha/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Applications in a host project and location.

      Args:
        request: (ApphubProjectsLocationsApplicationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListApplicationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications',
        http_method='GET',
        method_id='apphub.projects.locations.applications.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/applications',
        request_field='',
        request_type_name='ApphubProjectsLocationsApplicationsListRequest',
        response_type_name='ListApplicationsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsApplicationsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}',
        http_method='PATCH',
        method_id='apphub.projects.locations.applications.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='application',
        request_type_name='ApphubProjectsLocationsApplicationsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (ApphubProjectsLocationsApplicationsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}:setIamPolicy',
        http_method='POST',
        method_id='apphub.projects.locations.applications.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='ApphubProjectsLocationsApplicationsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (ApphubProjectsLocationsApplicationsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/applications/{applicationsId}:testIamPermissions',
        http_method='POST',
        method_id='apphub.projects.locations.applications.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='ApphubProjectsLocationsApplicationsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsDiscoveredServicesService(base_api.BaseApiService):
    """Service class for the projects_locations_discoveredServices resource."""

    _NAME = 'projects_locations_discoveredServices'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsDiscoveredServicesService, self).__init__(client)
      self._upload_configs = {
          }

    def FindUnregistered(self, request, global_params=None):
      r"""Finds unregistered services in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredServicesFindUnregisteredRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FindUnregisteredServicesResponse) The response message.
      """
      config = self.GetMethodConfig('FindUnregistered')
      return self._RunMethod(
          config, request, global_params=global_params)

    FindUnregistered.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredServices:findUnregistered',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredServices.findUnregistered',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/discoveredServices:findUnregistered',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredServicesFindUnregisteredRequest',
        response_type_name='FindUnregisteredServicesResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a Discovered Service in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredServicesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DiscoveredService) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredServices/{discoveredServicesId}',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredServices.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredServicesGetRequest',
        response_type_name='DiscoveredService',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Discovered Services that can be added to an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredServicesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListDiscoveredServicesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredServices',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredServices.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/discoveredServices',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredServicesListRequest',
        response_type_name='ListDiscoveredServicesResponse',
        supports_download=False,
    )

    def Lookup(self, request, global_params=None):
      r"""Lists a Discovered Service in a host project and location, with a given resource URI.

      Args:
        request: (ApphubProjectsLocationsDiscoveredServicesLookupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (LookupDiscoveredServiceResponse) The response message.
      """
      config = self.GetMethodConfig('Lookup')
      return self._RunMethod(
          config, request, global_params=global_params)

    Lookup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredServices:lookup',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredServices.lookup',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['uri'],
        relative_path='v1alpha/{+parent}/discoveredServices:lookup',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredServicesLookupRequest',
        response_type_name='LookupDiscoveredServiceResponse',
        supports_download=False,
    )

  class ProjectsLocationsDiscoveredWorkloadsService(base_api.BaseApiService):
    """Service class for the projects_locations_discoveredWorkloads resource."""

    _NAME = 'projects_locations_discoveredWorkloads'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsDiscoveredWorkloadsService, self).__init__(client)
      self._upload_configs = {
          }

    def FindUnregistered(self, request, global_params=None):
      r"""Finds unregistered workloads in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredWorkloadsFindUnregisteredRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FindUnregisteredWorkloadsResponse) The response message.
      """
      config = self.GetMethodConfig('FindUnregistered')
      return self._RunMethod(
          config, request, global_params=global_params)

    FindUnregistered.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredWorkloads:findUnregistered',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredWorkloads.findUnregistered',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/discoveredWorkloads:findUnregistered',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredWorkloadsFindUnregisteredRequest',
        response_type_name='FindUnregisteredWorkloadsResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a Discovered Workload in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredWorkloadsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DiscoveredWorkload) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredWorkloads/{discoveredWorkloadsId}',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredWorkloads.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredWorkloadsGetRequest',
        response_type_name='DiscoveredWorkload',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Discovered Workloads that can be added to an Application in a host project and location.

      Args:
        request: (ApphubProjectsLocationsDiscoveredWorkloadsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListDiscoveredWorkloadsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredWorkloads',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredWorkloads.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/discoveredWorkloads',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredWorkloadsListRequest',
        response_type_name='ListDiscoveredWorkloadsResponse',
        supports_download=False,
    )

    def Lookup(self, request, global_params=None):
      r"""Lists a Discovered Workload in a host project and location, with a given resource URI.

      Args:
        request: (ApphubProjectsLocationsDiscoveredWorkloadsLookupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (LookupDiscoveredWorkloadResponse) The response message.
      """
      config = self.GetMethodConfig('Lookup')
      return self._RunMethod(
          config, request, global_params=global_params)

    Lookup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/discoveredWorkloads:lookup',
        http_method='GET',
        method_id='apphub.projects.locations.discoveredWorkloads.lookup',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['uri'],
        relative_path='v1alpha/{+parent}/discoveredWorkloads:lookup',
        request_field='',
        request_type_name='ApphubProjectsLocationsDiscoveredWorkloadsLookupRequest',
        response_type_name='LookupDiscoveredWorkloadResponse',
        supports_download=False,
    )

  class ProjectsLocationsExtendedMetadataSchemasService(base_api.BaseApiService):
    """Service class for the projects_locations_extendedMetadataSchemas resource."""

    _NAME = 'projects_locations_extendedMetadataSchemas'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsExtendedMetadataSchemasService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets an Extended Metadata Schema.

      Args:
        request: (ApphubProjectsLocationsExtendedMetadataSchemasGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExtendedMetadataSchema) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/extendedMetadataSchemas/{extendedMetadataSchemasId}',
        http_method='GET',
        method_id='apphub.projects.locations.extendedMetadataSchemas.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsExtendedMetadataSchemasGetRequest',
        response_type_name='ExtendedMetadataSchema',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Extended Metadata Schemas available in a host project and location.

      Args:
        request: (ApphubProjectsLocationsExtendedMetadataSchemasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExtendedMetadataSchemasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/extendedMetadataSchemas',
        http_method='GET',
        method_id='apphub.projects.locations.extendedMetadataSchemas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/extendedMetadataSchemas',
        request_field='',
        request_type_name='ApphubProjectsLocationsExtendedMetadataSchemasListRequest',
        response_type_name='ListExtendedMetadataSchemasResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (ApphubProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='apphub.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='ApphubProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (ApphubProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='apphub.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (ApphubProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='apphub.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (ApphubProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='apphub.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1alpha/{+name}/operations',
        request_field='',
        request_type_name='ApphubProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServiceProjectAttachmentsService(base_api.BaseApiService):
    """Service class for the projects_locations_serviceProjectAttachments resource."""

    _NAME = 'projects_locations_serviceProjectAttachments'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsServiceProjectAttachmentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Attaches a service project to the host project.

      Args:
        request: (ApphubProjectsLocationsServiceProjectAttachmentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/serviceProjectAttachments',
        http_method='POST',
        method_id='apphub.projects.locations.serviceProjectAttachments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId', 'serviceProjectAttachmentId'],
        relative_path='v1alpha/{+parent}/serviceProjectAttachments',
        request_field='serviceProjectAttachment',
        request_type_name='ApphubProjectsLocationsServiceProjectAttachmentsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a service project attachment.

      Args:
        request: (ApphubProjectsLocationsServiceProjectAttachmentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/serviceProjectAttachments/{serviceProjectAttachmentsId}',
        http_method='DELETE',
        method_id='apphub.projects.locations.serviceProjectAttachments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsServiceProjectAttachmentsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a service project attachment.

      Args:
        request: (ApphubProjectsLocationsServiceProjectAttachmentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ServiceProjectAttachment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/serviceProjectAttachments/{serviceProjectAttachmentsId}',
        http_method='GET',
        method_id='apphub.projects.locations.serviceProjectAttachments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsServiceProjectAttachmentsGetRequest',
        response_type_name='ServiceProjectAttachment',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists service projects attached to the host project.

      Args:
        request: (ApphubProjectsLocationsServiceProjectAttachmentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListServiceProjectAttachmentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/serviceProjectAttachments',
        http_method='GET',
        method_id='apphub.projects.locations.serviceProjectAttachments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/serviceProjectAttachments',
        request_field='',
        request_type_name='ApphubProjectsLocationsServiceProjectAttachmentsListRequest',
        response_type_name='ListServiceProjectAttachmentsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def DetachServiceProjectAttachment(self, request, global_params=None):
      r"""Detaches a service project from a host project. You can call this API from any service project without needing access to the host project that it is attached to.

      Args:
        request: (ApphubProjectsLocationsDetachServiceProjectAttachmentRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DetachServiceProjectAttachmentResponse) The response message.
      """
      config = self.GetMethodConfig('DetachServiceProjectAttachment')
      return self._RunMethod(
          config, request, global_params=global_params)

    DetachServiceProjectAttachment.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}:detachServiceProjectAttachment',
        http_method='POST',
        method_id='apphub.projects.locations.detachServiceProjectAttachment',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:detachServiceProjectAttachment',
        request_field='detachServiceProjectAttachmentRequest',
        request_type_name='ApphubProjectsLocationsDetachServiceProjectAttachmentRequest',
        response_type_name='DetachServiceProjectAttachmentResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (ApphubProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='apphub.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def GetBoundary(self, request, global_params=None):
      r"""Gets a Boundary.

      Args:
        request: (ApphubProjectsLocationsGetBoundaryRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Boundary) The response message.
      """
      config = self.GetMethodConfig('GetBoundary')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetBoundary.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/boundary',
        http_method='GET',
        method_id='apphub.projects.locations.getBoundary',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ApphubProjectsLocationsGetBoundaryRequest',
        response_type_name='Boundary',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (ApphubProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='apphub.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='ApphubProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

    def LookupServiceProjectAttachment(self, request, global_params=None):
      r"""Lists a service project attachment for a given service project. You can call this API from any project to find if it is attached to a host project.

      Args:
        request: (ApphubProjectsLocationsLookupServiceProjectAttachmentRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (LookupServiceProjectAttachmentResponse) The response message.
      """
      config = self.GetMethodConfig('LookupServiceProjectAttachment')
      return self._RunMethod(
          config, request, global_params=global_params)

    LookupServiceProjectAttachment.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}:lookupServiceProjectAttachment',
        http_method='GET',
        method_id='apphub.projects.locations.lookupServiceProjectAttachment',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:lookupServiceProjectAttachment',
        request_field='',
        request_type_name='ApphubProjectsLocationsLookupServiceProjectAttachmentRequest',
        response_type_name='LookupServiceProjectAttachmentResponse',
        supports_download=False,
    )

    def UpdateBoundary(self, request, global_params=None):
      r"""Updates a Boundary.

      Args:
        request: (ApphubProjectsLocationsUpdateBoundaryRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('UpdateBoundary')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateBoundary.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/boundary',
        http_method='PATCH',
        method_id='apphub.projects.locations.updateBoundary',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='boundary',
        request_type_name='ApphubProjectsLocationsUpdateBoundaryRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(ApphubV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of migration job object lookup command."""


from googlecloudsdk.api_lib.database_migration import objects
from googlecloudsdk.api_lib.database_migration import resource_args
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.database_migration.objects import flags as objects_flags

DESCRIPTION = (
    'Lookup a migration job object by its source object identifier (e.g.'
    ' database)'
)
EXAMPLES = """\
    To lookup an existing migration job object:

        $ {command} --migration-job=my-job --location=us-central1 --database=my-database
   """


@base.RegionalEndpointsSupported
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Lookup(base.Command):
  """Lookup a DMS migration job object."""

  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES}

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    resource_args.AddOnlyMigrationJobResourceArgs(
        parser, 'to list migration job objects', positional=False
    )
    objects_flags.AddSourceObjectIdentifierFlag(parser)

  def Run(self, args):
    """Lookup a DMS migration job object.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the looked up migration job object if the
      lookup was successful.
    """
    migration_job_ref = args.CONCEPTS.migration_job.Parse()
    objects_client = objects.ObjectsClient(
        self.ReleaseTrack(),
        location=migration_job_ref.locationsId,
    )

    return objects_client.Lookup(migration_job_ref, args)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing Device Run operations."""

from googlecloudsdk.api_lib import device_run
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.device_run import resource_args
from googlecloudsdk.core import resources


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class List(base.ListCommand):
  """List all Device Run operations."""

  @staticmethod
  def Args(parser):
    resource_args.AddLocationResourceArg(parser, 'list operations')
    parser.display_info.AddFormat(
        'table(name.basename():label=OPERATION_ID, done)'
    )

    def UriFunc(resource):
      ref = resources.REGISTRY.Parse(
          resource.name, collection='devicerun.projects.locations.operations'
      )
      return ref.SelfLink()

    parser.display_info.AddUriFunc(UriFunc)

  def Run(self, args):
    location_ref = args.CONCEPTS.location.Parse()
    client = device_run.OperationsClient(api_version='v1alpha')
    return list(client.List(location_ref, limit=args.limit))


List.detailed_help = {
    'DESCRIPTION': 'List all Device Run operations.',
    'EXAMPLES': (
        """\
The following command lists all Device Run operations in us-central1:

  $ {command} --location=us-central1
"""
    ),
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Build utils."""

import os.path
import re

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.api_lib.iam import util as iam_api_util
from googlecloudsdk.api_lib.run import service as service_lib
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.run import config_changes
from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.core import log

_LEGACY_BUILD_SA_FORMAT = r'^\d+@cloudbuild\.gserviceaccount\.com$'
_PROJECT_TOML_FILE_NAME = 'project.toml'


def _GetDefaultBuildServiceAccount(project_id, region='global'):
  """Gets the default build service account for a project."""
  client = cloudbuild_util.GetClientInstance(location=region)
  name = f'projects/{project_id}/locations/{region}/defaultServiceAccount'
  return client.projects_locations.GetDefaultServiceAccount(
      client.MESSAGES_MODULE.CloudbuildProjectsLocationsGetDefaultServiceAccountRequest(
          name=name
      )
  ).serviceAccountEmail


def _ExtractServiceAccountEmail(service_account):
  """Extracts the service account email from the service account resource."""
  match = re.search(r'/serviceAccounts/([^/]+)$', service_account)
  if match:
    return match.group(1)
  else:
    return None


def _DescribeServiceAccount(service_account_email):
  client, messages = iam_api_util.GetClientAndMessages()
  return client.projects_serviceAccounts.Get(
      messages.IamProjectsServiceAccountsGetRequest(
          name=iam_util.EmailToAccountResourceName(service_account_email)
      )
  )


def ValidateBuildServiceAccountAndPromptWarning(
    project_id, region, build_service_account=None
):
  """Util to validate the default build service account permission.

  Prompt a warning to users if cloudbuild.builds.builder is missing.

  Args:
    project_id: id of project.
    region: region to deploy the service.
    build_service_account: user provided build service account. It will be None,
      if user doesn't provide it.

  Raises:
    ServiceAccountError: if the build service account is disabled/not
    found/missing required permissions.
  """

  if build_service_account is None:
    build_service_account = _GetDefaultBuildServiceAccount(project_id, region)
  service_account_email = _ExtractServiceAccountEmail(build_service_account)

  try:
    if not re.match(_LEGACY_BUILD_SA_FORMAT, service_account_email):
      build_service_account_description = _DescribeServiceAccount(
          service_account_email
      )
      if build_service_account_description.disabled:
        raise serverless_exceptions.ServiceAccountError(
            'Could not build the function due to disabled service account used'
            ' by Cloud Build. Please make sure that the service account:'
            f' [{build_service_account}] is active.'
        )
  except apitools_exceptions.HttpForbiddenError:
    # Just show a warning but not breaking the deployment.
    # We are doing best effort here.
    log.warning(
        'Your account does not have permission to check details of build'
        f' service account {build_service_account}. If the deployment fails,'
        f' please ensure {build_service_account} is active.'
    )
  except apitools_exceptions.HttpNotFoundError as exc:
    log.warning(
        f'The build service account {build_service_account} does not exist. If'
        ' you just created this account, or if this is your first time'
        ' deploying with the default build service account, it may take a few'
        ' minutes for the service account to become fully available. Please'
        ' try again later.'
    )
    raise serverless_exceptions.ServiceAccountError(
        f'Build service account {build_service_account} does not exist.'
    ) from exc


def CreateBuildPack(container):
  """A helper method to configure buildpack."""
  pack = [{'image': container.image}]
  changes = []
  source = container.source
  project_toml_file = source + '/' + _PROJECT_TOML_FILE_NAME
  command_arg = getattr(container, 'command', None)
  function_arg = getattr(container, 'function', None)
  if command_arg is not None:
    command = ' '.join(command_arg)
    pack[0].update(
        {'envs': ['GOOGLE_ENTRYPOINT="{command}"'.format(command=command)]}
    )
  elif function_arg is not None:
    pack[0].update({
        'envs': [
            'GOOGLE_FUNCTION_SIGNATURE_TYPE=http',
            'GOOGLE_FUNCTION_TARGET={target}'.format(target=function_arg),
        ]
    })
  if os.path.exists(project_toml_file):
    pack[0].update({'project_descriptor': _PROJECT_TOML_FILE_NAME})
  return pack, changes


def GetBuildWorkerPool(args, annotated_build_worker_pool, service, changes):
  """Gets the build worker pool from user flags and annotations.

  Args:
    args: argparse.Namespace, Command line arguments
    annotated_build_worker_pool: Build worker pool value from service
      annotations.
    service: Existing Cloud Run service.
    changes: List of config changes.

  Returns:
    build_worker_pool value or
    None meaning clear-worker-pool flag was set
    or build-worker-pool was an empty string.
  """
  if _ShouldClearBuildWorkerPool(args):
    worker_pool_key = service_lib.RUN_FUNCTIONS_BUILD_WORKER_POOL_ANNOTATION
    if service and service.annotations.get(worker_pool_key):
      changes.append(config_changes.DeleteAnnotationChange(worker_pool_key))
    return None
  return (
      args.build_worker_pool
      if flags.FlagIsExplicitlySet(args, 'build_worker_pool')
      else annotated_build_worker_pool
  )


def _ShouldClearBuildWorkerPool(args):
  return flags.FlagIsExplicitlySet(args, 'clear_build_worker_pool') or (
      flags.FlagIsExplicitlySet(args, 'build_worker_pool')
      and not args.build_worker_pool
  )


def GetBuildServiceAccount(
    args, annotated_build_service_account, container, service, changes
):
  """Returns cloud build service account.

  Args:
    args: argparse.Namespace, Command line arguments
    annotated_build_service_account: string. The build service account annotated
      on the service used by cloud run functions.
    container: Container. The container to deploy.
    service: Service. The service being changed.
    changes: List of config changes.

  Returns:
    build service account value where
    None means there were no annotations, user specified to clear the
    build service account, or the build service account was an empty string.
  """
  build_sa_key = service_lib.RUN_FUNCTIONS_BUILD_SERVICE_ACCOUNT_ANNOTATION
  build_service_account = getattr(container, 'build_service_account', None)
  if _ShouldClearBuildServiceAccount(args, build_service_account):
    if service and service.annotations.get(build_sa_key):
      changes.append(config_changes.DeleteAnnotationChange(build_sa_key))
    return None
  return build_service_account or annotated_build_service_account


def _ShouldClearBuildServiceAccount(args, build_service_account):
  if flags.FlagIsExplicitlySet(args, 'clear_build_service_account'):
    return True
  if (
      flags.FlagIsExplicitlySet(args, 'build_service_account')
      and not build_service_account
  ):
    return True
  return False




"""Generated client library for lustre version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.lustre.v1alpha import lustre_v1alpha_messages as messages


class LustreV1alpha(base_api.BaseApiClient):
  """Generated client library for service lustre version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://lustre.googleapis.com/'
  MTLS_BASE_URL = 'https://lustre.mtls.googleapis.com/'

  _PACKAGE = 'lustre'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'LustreV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new lustre handle."""
    url = url or self.BASE_URL
    super(LustreV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_instances_directoryPolicies = self.ProjectsLocationsInstancesDirectoryPoliciesService(self)
    self.projects_locations_instances = self.ProjectsLocationsInstancesService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsInstancesDirectoryPoliciesService(base_api.BaseApiService):
    """Service class for the projects_locations_instances_directoryPolicies resource."""

    _NAME = 'projects_locations_instances_directoryPolicies'

    def __init__(self, client):
      super(LustreV1alpha.ProjectsLocationsInstancesDirectoryPoliciesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a directory policy resource.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies',
        http_method='POST',
        method_id='lustre.projects.locations.instances.directoryPolicies.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['directoryPolicyId'],
        relative_path='v1alpha/{+parent}/directoryPolicies',
        request_field='directoryPolicy',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a directory policy resource.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies/{directoryPoliciesId}',
        http_method='DELETE',
        method_id='lustre.projects.locations.instances.directoryPolicies.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single directory policy.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DirectoryPolicy) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies/{directoryPoliciesId}',
        http_method='GET',
        method_id='lustre.projects.locations.instances.directoryPolicies.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest',
        response_type_name='DirectoryPolicy',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies/{directoryPoliciesId}:getIamPolicy',
        http_method='GET',
        method_id='lustre.projects.locations.instances.directoryPolicies.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1alpha/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Gets details of multiple directory policies under a given instance.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListDirectoryPoliciesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies',
        http_method='GET',
        method_id='lustre.projects.locations.instances.directoryPolicies.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/directoryPolicies',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesListRequest',
        response_type_name='ListDirectoryPoliciesResponse',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies/{directoryPoliciesId}:setIamPolicy',
        http_method='POST',
        method_id='lustre.projects.locations.instances.directoryPolicies.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/directoryPolicies/{directoryPoliciesId}:testIamPermissions',
        http_method='POST',
        method_id='lustre.projects.locations.instances.directoryPolicies.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsInstancesService(base_api.BaseApiService):
    """Service class for the projects_locations_instances resource."""

    _NAME = 'projects_locations_instances'

    def __init__(self, client):
      super(LustreV1alpha.ProjectsLocationsInstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new instance in a given project and location.

      Args:
        request: (LustreProjectsLocationsInstancesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances',
        http_method='POST',
        method_id='lustre.projects.locations.instances.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['instanceId', 'requestId'],
        relative_path='v1alpha/{+parent}/instances',
        request_field='instance',
        request_type_name='LustreProjectsLocationsInstancesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single instance.

      Args:
        request: (LustreProjectsLocationsInstancesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='DELETE',
        method_id='lustre.projects.locations.instances.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force', 'requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ExportData(self, request, global_params=None):
      r"""Exports data from a Managed Lustre instance to Cloud Storage.

      Args:
        request: (LustreProjectsLocationsInstancesExportDataRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ExportData')
      return self._RunMethod(
          config, request, global_params=global_params)

    ExportData.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:exportData',
        http_method='POST',
        method_id='lustre.projects.locations.instances.exportData',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:exportData',
        request_field='exportDataRequest',
        request_type_name='LustreProjectsLocationsInstancesExportDataRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single instance.

      Args:
        request: (LustreProjectsLocationsInstancesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Instance) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='GET',
        method_id='lustre.projects.locations.instances.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesGetRequest',
        response_type_name='Instance',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (LustreProjectsLocationsInstancesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:getIamPolicy',
        http_method='GET',
        method_id='lustre.projects.locations.instances.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1alpha/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def ImportData(self, request, global_params=None):
      r"""Imports data from Cloud Storage to a Managed Lustre instance.

      Args:
        request: (LustreProjectsLocationsInstancesImportDataRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ImportData')
      return self._RunMethod(
          config, request, global_params=global_params)

    ImportData.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:importData',
        http_method='POST',
        method_id='lustre.projects.locations.instances.importData',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:importData',
        request_field='importDataRequest',
        request_type_name='LustreProjectsLocationsInstancesImportDataRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists instances in a given project and location.

      Args:
        request: (LustreProjectsLocationsInstancesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListInstancesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances',
        http_method='GET',
        method_id='lustre.projects.locations.instances.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/instances',
        request_field='',
        request_type_name='LustreProjectsLocationsInstancesListRequest',
        response_type_name='ListInstancesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single instance.

      Args:
        request: (LustreProjectsLocationsInstancesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='PATCH',
        method_id='lustre.projects.locations.instances.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='instance',
        request_type_name='LustreProjectsLocationsInstancesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RescheduleMaintenance(self, request, global_params=None):
      r"""Reschedules a planned maintenance event for a specific instance.

      Args:
        request: (LustreProjectsLocationsInstancesRescheduleMaintenanceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RescheduleMaintenance')
      return self._RunMethod(
          config, request, global_params=global_params)

    RescheduleMaintenance.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:rescheduleMaintenance',
        http_method='POST',
        method_id='lustre.projects.locations.instances.rescheduleMaintenance',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:rescheduleMaintenance',
        request_field='rescheduleMaintenanceRequest',
        request_type_name='LustreProjectsLocationsInstancesRescheduleMaintenanceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (LustreProjectsLocationsInstancesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:setIamPolicy',
        http_method='POST',
        method_id='lustre.projects.locations.instances.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='LustreProjectsLocationsInstancesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (LustreProjectsLocationsInstancesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:testIamPermissions',
        http_method='POST',
        method_id='lustre.projects.locations.instances.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='LustreProjectsLocationsInstancesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(LustreV1alpha.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (LustreProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='lustre.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='LustreProjectsLocationsOperationsCancelRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (LustreProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='lustre.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsOperationsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (LustreProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='lustre.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (LustreProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='lustre.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1alpha/{+name}/operations',
        request_field='',
        request_type_name='LustreProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(LustreV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (LustreProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='lustre.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='LustreProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (LustreProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='lustre.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='LustreProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(LustreV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }
